// http://blog.doityourselfandroid.com/2011/08/06/oauth-2-0-flow-android/
// http://code.google.com/p/google-api-java-client/wiki/OAuth2Draft10

package org.average.whereami;

import java.io.IOException;

import org.average.whereami.ClientCredentials;
import org.average.whereami.PersistentStore;

import android.app.Activity;
import android.os.Bundle;
import android.graphics.Bitmap;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.google.api.client.auth.oauth2.draft10.AccessTokenResponse;
import com.google.api.client.googleapis.auth.oauth2.draft10.GoogleAccessTokenRequest.GoogleAuthorizationCodeGrant;
import com.google.api.client.googleapis.auth.oauth2.draft10.GoogleAuthorizationRequestUrl;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.jackson.JacksonFactory;

public class Authorize extends Activity {

    final String TAG = getClass().getName();
    SharedPreferences prefs;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.w(TAG, "Starting task to retrieve request token.");
        prefs = PreferenceManager.getDefaultSharedPreferences(this);
    }

    @Override
    protected void onResume() {
        super.onResume();
        WebView webview = new WebView(this);
        webview.getSettings().setJavaScriptEnabled(true);
        webview.setVisibility(View.VISIBLE);
        setContentView(webview);
        String authorizationUrl = new GoogleAuthorizationRequestUrl(
                                        ClientCredentials.CLIENT_ID,
                                        ClientCredentials.REDIRECT_URI,
                                        ClientCredentials.SCOPE).build();
        webview.setWebViewClient(new WebViewClient() {
            @Override
            public void onPageStarted(WebView view, String url, Bitmap bitmap) {
                Log.w(TAG, "onPageStarted : " + url);
            }
            @Override
            public void onPageFinished(WebView view, String url) {
                if (url.startsWith(ClientCredentials.REDIRECT_URI)) {
                    try {
                        if (url.indexOf("code=")!=-1) {
                            String code = extractCodeFromUrl(url);
                            AccessTokenResponse accessTokenResponse =
                              new GoogleAuthorizationCodeGrant(
                                new NetHttpTransport(),
                                new JacksonFactory(),
                                ClientCredentials.CLIENT_ID,
                                ClientCredentials.CLIENT_SECRET,
                                code,
                                ClientCredentials.REDIRECT_URI).execute();
                            PersistentStore store = new PersistentStore(prefs);
                            store.writeTokens(accessTokenResponse);
                            finish();
                        } else if (url.indexOf("error=")!=-1) {
                            new PersistentStore(prefs).clearTokens();
                            finish();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
                Log.w(TAG, "onPageFinished : " + url);
            }

            private String extractCodeFromUrl(String url) {
                return url.substring(
                            ClientCredentials.REDIRECT_URI.length()+7,
                            url.length());
            }
        });

        webview.loadUrl(authorizationUrl);
    }
}
